/***************************************************************************
 *
 * Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: proxy.c,v 1.31 2000/12/27 14:42:34 bazsi Exp $
 *
 * Author  : Bazsi
 * Auditor : kisza
 * Last version: 1.23
 * Notes:
 *
 ***************************************************************************/

#include <zorp/proxy.h>
#include <zorp/policy.h>
#include <zorp/pyvars.h>
#include <zorp/pystream.h>
#include <zorp/log.h>

#include <stdarg.h>
#include <sys/socket.h>

#define CORE_DEBUG "core.debug"

/* a two-way connection between streams */

/* z_proxy_var_new() should be here, but is implemented in pyvars.c for now */

/*+

  Initialize a given proxy instance.

  Parameters:
    self       proxy instance to initialize
    session_id session_id
    client     client side Stream
    handler    policy object to send events to

  Returns:
    none

  +*/
void
z_proxy_init(ZProxy *self, 
             gchar *session_id, 
	     ZStream *client, 
	     ZPolicyObj *handler)
{
  self->endpoints[EP_CLIENT] = client;
  z_stream_ref(client);
  
  strncpy(self->session_id, session_id, MAX_SESSION_ID);
  self->handler = handler;
  z_policy_var_ref(handler);
  self->setattr = z_pyvars_setattr;
  self->getattr = z_pyvars_getattr;
  self->vars = z_pyvars_new();
  self->thread = PyThreadState_New(z_python_global_state->interp);
}

/*+

  Called to free up a proxy instance. Frees up associated
  resources, closes streams, and calls the free virtual method.

  Parameters:
    self         proxy instance

  Returns:
    none

  +*/
void
z_proxy_destroy(ZProxy *self)
{
  int i;

  z_proxy_log(self, CORE_DEBUG, 5, "session_stop");
  z_policy_lock(self->thread);
  z_policy_var_unref(self->handler);  
  z_policy_unlock(self->thread);
  self->handler = NULL;
  for (i = EP_CLIENT; i <= EP_SERVER; i++)
    {
      if (self->endpoints[i])
        {
          z_stream_shutdown(self->endpoints[i], SHUT_RDWR);
          z_stream_close(self->endpoints[i]);
          z_stream_unref(self->endpoints[i]);
          self->endpoints[i] = NULL;
        }
    }
  if (self->free)
    (*self->free)(self);
  z_python_lock();
  PyThreadState_Clear(self->thread);
  z_python_unlock();
  z_pyvars_destroy(self->vars);
  g_free(self);
}

void
z_proxy_quit(ZProxy *self)
{
  self->flags |= PF_QUIT;
}

/*+

  Send a config event to the associated policy object.

  Parameters:
    self      proxy instance

  Returns:
    none

  Remarks:
    Locks the policy engine.

  +*/
void
z_proxy_config_event(ZProxy *self)
{
  ZPolicyObj *res;
  
  z_policy_lock(self->thread);
  res = z_policy_call(self->handler, "config", NULL);
  z_policy_var_unref(res);
  z_policy_unlock(self->thread);
}

/*+

  Send a startup event to the associated policy object.

  Parameters:
    self      proxy instance

  Returns:
    none

  Remarks:
    Locks the policy engine.

  +*/
void
z_proxy_startup_event(ZProxy *self)
{
  ZPolicyObj *res;

  z_policy_lock(self->thread);
  res = z_policy_call(self->handler, "startUp", NULL);
  z_policy_var_unref(res);
  z_policy_unlock(self->thread);  
}

/*+

  Send a shutdown event to the associated policy object.

  Parameters:
    self      proxy instance

  Returns:
    none

  Remarks:
    Locks the policy engine.

  +*/
void 
z_proxy_shutdown_event(ZProxy *self)
{
  ZPolicyObj *res;

  z_policy_lock(self->thread);
  res = z_policy_call(self->handler, "shutDown", NULL);
  z_policy_var_unref(res);
  z_policy_unlock(self->thread);
}

/*+

  Send a connectServer event to the associated policy object.

  Parameters:
    self      proxy instance
    host      host to connect to, used as a hint by the 
              policy layer but may as well be ignored
    port      port in host to connect to

  Returns:
    TRUE if the server-side connection is established, otherwise the
    connection should be closed.

  Remarks:
    Locks the policy engine.

  +*/
gint
z_proxy_connect_server_event(ZProxy *self, char *host, gint port)
{
  ZPolicyObj *args, *res;
  gint rc;
  
  z_policy_lock(self->thread);
  self->endpoints[EP_SERVER] = NULL;
  if (host)
    args = z_policy_var_build("(si)", host, port);
  else
    {
      Py_XINCREF(Py_None);
      args = z_policy_var_build("(Oi)", Py_None, port);
    }
  res = z_policy_call(self->handler, "connectServer", args);
  
  z_policy_var_unref(args);

  if (res && z_py_zorp_stream_check(res))
    {
      self->endpoints[EP_SERVER] = ((ZorpStream *) res)->stream;
      z_stream_ref(self->endpoints[EP_SERVER]);
      rc = 1;
    }
  else
    {
      rc = 0;
    }
  z_policy_var_unref(res);
  z_policy_unlock(self->thread);
  return rc;
}
