############################################################################
##
## Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##
## $Id: Domain.py,v 1.6 2000/12/30 13:11:52 bazsi Exp $
##
############################################################################

from SockAddr import SockAddrInet, inet_ntoa, inet_aton
from string import split, atoi
from socket import ntohl, htonl

class Domain:
	"""Abstract base class for address domains.
	
	An address domain encapsulates an address type (AF_INET, AF_INET6
	etc) and provides functions to parse these addresses.
	   
	Attributes:
	
	  family  -- address family this domain uses
	"""
	family = 0
	def __init__(self):
		pass

	def __cmp__(self, other):
		raise NotImplementedError

class InetDomain(Domain):
	"""

	A class representing internet addresses.  The inet objects are
	comparable, comparison means "contains", "equal to" and
	"contained by". The comparison can raise ValueError for
	incomparable ip addresses.
		
	Attributes:
	
	  mask_bits -- number of bits in the netmask
	  
	  mask	    -- netmask in network byte order
	  
	  ip        -- network addresss in network byte order
		       
       	"""
	def __init__(self, addr):
		"""Initializes an InetDomain instance
		
		Parses the argument addr and fills in attributes
		accordingly.
		
		Arguments:
		
		  addr -- the string representation of an address, or 
		          address range
	       	"""
	        #FIXME: input validation!
	        self.family = 2
		parts = split(addr,'/')
		try:
			self.mask_bits = atoi(parts[1])
			self.mask = htonl(((1 << self.mask_bits) - 1) << (32 - self.mask_bits))
		except IndexError:
			self.mask_bits = 32
			self.mask = 0xffffffff
		self.ip = inet_aton(parts[0]) & self.mask

	def __str__(self):
		"""Returns the string representation of this instance.
		
                Returns the string representation of the address/mask
                
                Arguments:
                
                  self -- this instance
                        
		Returns:
		
                  string
                """
                return "%s/%u" % (inet_ntoa(self.ip), self.mask_bits)

	def __cmp__(self, other):
		"""Compare this instance to another.
		
		Compare this instance to another InetDomain instance or
		to a SockAddrInet instance.
		   
		Arguments:
		
		  self  -- this instance
		  
		  other -- the other InetDomain object to compare to
			
		Exceptions: 		
		
		  Raises ValueError if self and other
		  are incomparable
		  
		Notes:
		
		  standard comparison operator, using
		  set inclusion on addresses
				
		  looks into the netmask and then the ip
		  numbers masked with the lesser mask if
		  AttributeError is raised, it assumes
		  that self or other is not of Inet
		  type.
		
				
		Returns:
		
		  -1, 0, or 1
		"""
		if isinstance(other, InetDomain):
			if ((self.ip == other.ip) & (self.mask == other.mask)):
				return 0
			if ((self.mask_bits >= other.mask_bits) & ((self.ip & other.netmask()) == other.ip)):
				return -1
			if ((other.mask_bits >= self.mask_bits) & ((other.ip & self.netmask()) == self.ip)):
				return 1
		else:
			try:
				if ((other.ip & self.netmask()) == self.ip):
					return 1
			except AttributeError:
			       	pass
		raise ValueError, '%s and %s are incomparable' % (self, other)
	
	def netaddr(self):
		"""Calculate the network address of this domain.
		
		Return the network address of this domain.
		   
		Arguments:
		
		  self -- this instance
		
		Returns:
		
		  ip address in network byte order

		Notes:

		  just returns the ip address, which is already masked
		  by the constructor
			
		"""
		return self.ip
	
	def broadcast(self):
		"""Calculate the broadcast address of this domain
		   
		Return the broadcast address of this domain calculated
		based on attributes.
		   
		Arguments:
		
		  self -- this instance

		Returns:
		
		  the broadcast address in network byte order
			
		Notes:
		   
			does bitwise or with the network address and the
			bits outside the netmask it handles the mask=0 case
			specially, because the '>>" semantics regarding the
			highest bit
			
		"""
		if self.mask_bits == 0:
			return self.ip
		
		return htonl(ntohl(self.ip) | (0x7fffffff >> (self.mask_bits - 1)))
		
	def netmask(self):
		"""Calculate netmask of this domain.
		
		Calculates and returns the netmask of this domain as an
		integer in network byte order.
		   
		Arguments:
		
		  self -- this instance
		
		Returns:
		
		  the network mask as ip in network byte order

		Notes:
		
		  rotate and mask
			
		  handles the mask=0 case specially, because the
		  rotation semantics regarding the highest bit
		"""
		return self.mask


class Inet6Domain(Domain):
	pass
