############################################################################
##
## Copyright (c) 2000 BalaBit IT Ltd, Budapest, Hungary
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##
## $Id: Plug.py,v 1.21 2000/12/27 14:42:34 bazsi Exp $
##
## Author  : Bazsi
## Auditor : kisza
## Last version: 1.10
## Notes:
##   - comments in English
##
############################################################################

from Zorp import *
import Service, Connector, Proxy, Session, Stream

PLUG_DEBUG = "plug.debug"

class PlugProxy(Proxy.Proxy):
	"""Wrapper for the builtin Plug proxy implemented in Zorp.
	
	Implements a general plug proxy, with optionally disabling
	data transfer in either direction.

	By default plug copies all data in both directions. To change
	this set the copy_to_client or copy_to_server attributes
	to FALSE.

	Note: Copying out of band data is not supported.

	Attributes:

	  copy_to_server -- Copy data in client->server direction
	  
	  copy_to_client -- Copy data in server->client direction
	  
	  bandwidth_to_client -- Readonly variable containing the utilized
                                 bandwidth in server->client direction.
				 
	  bandwidth_to_server -- Readonly variable containing the utilized
	                         bandwidth in client->server direction.
				 
	  packet_stats_interval -- The number of milliseconds between
	                           two successive packetStats() events.
				   By default: 0.
				   NOTE: this is currently implemented as the
				   number of passing packages, not as
				   milliseconds.
				   
	  stack_proxy -- the proxy class to stack into the connection

	Notes:

	  packetStats() event is called whenever the time interval
	  elapses (or as currently implemented the given number of
	  packets were transmitted). this event receives packet
	  statistics as parameters.

	  It's useful for terminating connections with excessive
	  bandwidth requirements (for instance to limit the impact of
	  the covert channel opened when using plug instead of a
	  protocol specific proxy)

	  packetStats is defined as:

	  def packetStats(self, client_bytes, client_pkts, server_bytes, server_pkts)
	  
	"""
	stack_proxy = None

	def __init__(self, session):
		"""Initialize a PlugProxy instance.

		Create and set up a PlugProxy instance.

		Arguments:

		  self -- this instance
		  
		  session -- session this instance belongs to
		
		"""
		Proxy.Proxy.__init__(self, "plug", session)

	def requestStack(self):
		"""Query if something is to be stacked into this plug.
		
		Callback called by the underlying C proxy to query if
		something is to be stacked.

		Arguments:
		
		  self -- this instance
		  
		Returns:
		
		  the class of the proxy to stack in
		"""
		if self.stack_proxy != None:
			return self.stackProxy
		return 0

	def stackProxy(self, client_fd, server_fd):
		"""Stack a proxy instance within this plug.
		
		Callback called by the underlying C proxy to actually
		stack in something.
		
		Arguments:

		  self -- this instance
		  
		  client_fd -- client fd
		  
		  server_fd -- server fd
		"""
		
		Proxy.proxyLog(self, PLUG_DEBUG, 7, "stackProxy called: %d, %d" % (client_fd, server_fd))
		subsession = Session.StackedSession(self)
		subsession.client_fd = client_fd
		subsession.client_stream = Stream.Stream(client_fd, "plug client upstream")
		subsession.server_fd = server_fd
		subsession.server_stream = Stream.Stream(server_fd, "plug server upstream")
		self.stack_proxy(subsession)
